 use reqwest::Client as HTTPClient;
use serde::Deserialize;
use serde_json::json;
use std::{collections::HashMap, env, error::Error};


// Структура для десериализации ответа от сервиса аутентификации
#[derive(Deserialize)]
struct CoreResponse {
    data: Option<Shout>,
}

#[derive(Deserialize)]
pub struct ShoutTopic {
    pub slug: String,
    pub title: String,
}

#[derive(Deserialize)]
pub struct ShoutAuthor {
    pub name: String,
}

#[derive(Deserialize)]
pub struct Shout {
    pub title: String,
    pub created_at: String,
    pub created_by: i32,
    pub main_topic: String,
    pub topics: Vec<ShoutTopic>,
    pub authors: Vec<ShoutAuthor>,
    pub layout: String,
}

pub async fn get_shout_by_id(shout_id: i32) -> Result<Shout, Box<dyn Error>> {
    let mut variables = HashMap::<String, i32>::new();
    let api_base = env::var("CORE_URL")?;
    let query_name = "get_shout";
    let operation = "GetShout";
    if shout_id != 0 {
        variables.insert("shout_id".to_string(), shout_id);
    }

    let gql = json!({
        "query": format!("query {}($slug: String, $shout_id: Int) {{ {}(slug: $slug, shout_id: $shout_id) {{ title created_at created_by topics {{ title slug }} authors {{ id name }} }} }}", operation, query_name),
        "operationName": operation,
        "variables": variables
    });

    let client = HTTPClient::new();
    let response = client
        .post(&api_base)
        .json(&gql)
        .send()
        .await?;

    if response.status().is_success() {
        let core_response: CoreResponse = response.json().await?;
        if let Some(shout) = core_response.data {
            return Ok(shout);
        }
        Err(Box::new(std::io::Error::new(
            std::io::ErrorKind::Other,
            "Shout not found",
        )))
    } else {
        Err(Box::new(std::io::Error::new(
            std::io::ErrorKind::Other,
            response.status().to_string(),
        )))
    }
}   